# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import re
# Logging
import logging
# Launch command
import subprocess as sp
# Functions and decorators
from functools import wraps

# Create logger for jplotlib
logger = logging.getLogger(__name__)


def jetson_clocks_checks(func):
    """ Enable and disable Jetson Clocks after NVP model change """
    @wraps(func)
    def wrapped(self, level):
        # Disable the jetson_clocks (only if is it active) before change NVPmodel level
        if self.jetson_clocks is not None:
            old_status = self.jetson_clocks.start
            if old_status:
                self.jetson_clocks.start = False
        status = func(self, level)
        # Enable again the jetson_clocks status
        if self.jetson_clocks is not None:
            if old_status:
                self.jetson_clocks.start = True
        # Return status function
        return status
    return wrapped


class NVPmodel():
    """
        This controller read the status from your NVIDIA Jetson and you can control
        performance and status.
        It is available for NVIDIA Jetson TX2, Jetson AGX Xavier, Jetson Nano
        Boards reference:
        * TX2: https://www.jetsonhacks.com/2017/03/25/nvpmodel-nvidia-jetson-tx2-development-kit/
        * AGX Xavier: https://www.jetsonhacks.com/2018/10/07/nvpmodel-nvidia-jetson-agx-xavier-developer-kit/
        * Nano: https://www.jetsonhacks.com/2019/04/10/jetson-nano-use-more-power/
    """
    REGEXP = re.compile(r'POWER_MODEL: ID=(.+?) NAME=((.*))')

    class NVPmodelException(Exception):
        pass

    def __init__(self, type_board, jetson_clocks=None):
        self.type_board = type_board
        self.jetson_clocks = jetson_clocks
        try:
            nvpmodel_p = sp.Popen(['nvpmodel', '-p', '--verbose'], stdout=sp.PIPE)
            out, _ = nvpmodel_p.communicate()
            # Log value
            logger.debug('nvqmodel status %s', out)
            # Decode lines
            lines = out.decode("utf-8")
            # Read all lines and extract modes
            self.board = []
            for line in lines.split("\n"):
                # Search configuration NVPmodel
                match = NVPmodel.REGEXP.search(line)
                # if match extract name and number
                if match:
                    pm = {"ID": int(match.group(1)), "Name": match.group(2), "status": True}
                    self.board += [pm]
        except OSError:
            logger.info("This board {} does not have NVP Model".format(type_board))
            raise NVPmodel.NVPmodelException("NVPmodel does not exist for this board {}".format(type_board))
        except AttributeError:
            logger.info("Wrong open")
            raise NVPmodel.NVPmodelException("Wrong open")
        # Initialize mode and num
        self.update()
        # Initialize with first configuration number
        self.selected = self.num

    @property
    def modes(self):
        return self.board

    @jetson_clocks_checks
    def set(self, level):
        """ Set nvpmodel to a new status """
        try:
            # Set the new nvpmodel status
            sep_nvp = sp.Popen(['nvpmodel', '-m', str(level)], stdout=sp.PIPE, stderr=sp.PIPE, stdin=sp.PIPE)
            out, _ = sep_nvp.communicate()
            if "NVPM ERROR" in out.decode("utf-8"):
                self.board[level]["status"] = False
                return False
            else:
                self.board[level]["status"] = True
                return True
        except OSError:
            logger.info("NVP Model does not exist")
            return False
        except AttributeError:
            logger.info("Wrong open")
            return False

    def increase(self):
        if self.selected + 1 < len(self.modes):
            self.selected += 1
            return self.set(self.selected)
        else:
            return False

    def decrease(self):
        if self.selected - 1 >= 0:
            self.selected -= 1
            return self.set(self.selected)
        else:
            return False

    def update(self):
        """ Read nvpmodel to know the status of the board """
        try:
            nvpmodel_p = sp.Popen(['nvpmodel', '-q'], stdout=sp.PIPE, stderr=sp.PIPE)
            out, _ = nvpmodel_p.communicate()
            # Log value
            logger.debug('nvqmodel status %s', out)
            # Decode lines and split
            lines = out.decode("utf-8").split("\n")
            # Return the mode type
            self.mode = str(lines[0].split(": ")[1])
            self.num = int(lines[1])
        except OSError:
            logger.info("NVP Model does not exist")
        except AttributeError:
            logger.info("Wrong open")
# EOF



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import curses
# Graphics elements
from .jtopguilib import (linear_percent_gauge,
                         make_gauge_from_percent,
                         plot_name_info,
                         draw_chart)


def GPU(stdscr, jetson, key):
    """
        Draw a plot with GPU payload
    """
    # Screen size
    max_y, max_x = stdscr.getmaxyx()
    # Evaluate size chart
    size_x = [2, max_x - 10]
    size_y = [1, max_y * 2 // 3 - 1]
    # Read GPU status
    if 'GR3D' in jetson.stats:
        gpu = jetson.stats['GR3D']
        # Draw the GPU chart
        draw_chart(stdscr, size_x, size_y, gpu, color=curses.color_pair(2))
        # Percent Gauge GPU
        linear_percent_gauge(stdscr, make_gauge_from_percent(jetson.stats['GR3D']), max_x // 2, offset=max_y * 2 // 3, start=2)
        # Temperature GPU
        if "GPU" in jetson.stats['temperatures']:
            plot_name_info(stdscr, max_y * 2 // 3, max_x // 2 + 4, "GPU Temp", jetson.stats['temperatures']['GPU']['text'])
        # Jetson clocks status
        jc = jetson.jetson_clocks
        if jc is not None:
            jc_status = jc.status
            if jc_status == "active":
                color = curses.color_pair(2)  # Running (Green)
            elif jc_status == "inactive":
                color = curses.A_NORMAL       # Normal (Grey)
            elif "ing" in jc_status:
                color = curses.color_pair(3)  # Warning (Yellow)
            else:
                color = curses.color_pair(1)  # Error (Red)
            # Show if JetsonClock is enabled or not
            if jc.enable:
                jc_status = "[" + jc_status + "]"
            plot_name_info(stdscr, max_y * 2 // 3 + 2, 2, "Jetson Clocks", jc_status, color)
        # NVP Model
        nvpmodel = jetson.nvpmodel
        if nvpmodel is not None:
            plot_name_info(stdscr, max_y * 2 // 3 + 3, 2, "NV Power[" + str(nvpmodel.num) + "]", nvpmodel.mode)



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
# Logging
import logging

from .core import NVPmodel
from .core import Tegrastats
from .core import Fan
from .core import JetsonClocks
from .core import (import_os_variables,
                   get_uptime,
                   status_disk,
                   get_local_interfaces)

# Create logger for jplotlib
logger = logging.getLogger(__name__)


class jtop():
    """
        JTOP is a complete controller of all systems in your NVIDIA Jetson
        * Tegrastats
        * NVP Model
        * Fan
    """

    class JtopException(Exception):
        pass

    # List of available fan
    JTOP_FOLDER = '/opt/jetson_stats/'
    LIST_FANS = ['/sys/kernel/debug/tegra_fan/', '/sys/devices/pwm-fan/']
    TEGRASTATS = ['/usr/bin/tegrastats', '/home/nvidia/tegrastats']

    def __init__(self, interval=500, time=10.0):
        # Load all Jetson variables
        if "JETSON_BOARD" not in os.environ:
            logger.info("Load jetson variables from script")
            for k, v in import_os_variables(jtop.JTOP_FOLDER + 'jetson_variables').items():
                logger.debug("New Enviroment variable {}:{}".format(k, v))
                os.environ[k] = v
        # Initialize jetson_clocks controller
        self.jc = JetsonClocks()
        # Initialize NVP model
        try:
            self.nvp = NVPmodel(os.environ["JETSON_TYPE"], jetson_clocks=self.jc)
        except NVPmodel.NVPmodelException:
            self.nvp = None
        # Find all fans availables
        self.qfan = None
        for path in jtop.LIST_FANS:
            try:
                self.qfan = Fan(path, interval, time)
                logger.info("Fan {} loaded!".format(path))
                break
            except Fan.FanException:
                logger.info("Fan {} not loaded".format(path))
        # Start process tegrastats
        tegrastats_file = ""
        for f_tegra in jtop.TEGRASTATS:
            if os.path.isfile(f_tegra):
                logger.info("Load tegrastats {}".format(f_tegra))
                tegrastats_file = f_tegra
                break
        if not tegrastats_file:
            raise jtop.JtopException("Tegrastats is not availabe on this board")
        # Initialize Tegrastats controller
        self._stats = {}
        self.tegrastats = Tegrastats(tegrastats_file, interval, time)

    @property
    def userid(self):
        return os.getuid()

    @property
    def fan(self):
        if self.qfan is not None:
            return self.qfan.status
        else:
            return None

    @property
    def disk(self):
        return status_disk()

    @property
    def jetson_clocks(self):
        return self.jc

    @property
    def uptime(self):
        return get_uptime()

    @property
    def nvpmodel(self):
        return self.nvp

    @property
    def local_interfaces(self):
        return get_local_interfaces()

    @property
    def board(self):
        board = {"name": os.environ["JETSON_DESCRIPTION"],
                 "type": os.environ["JETSON_TYPE"],
                 "jetpack": os.environ["JETSON_JETPACK"] + " [L4T " + os.environ["JETSON_L4T"] + "]",
                 "GPU-arch": os.environ["JETSON_CUDA_ARCH_BIN"]}
        libraries = {"CUDA": os.environ["JETSON_CUDA"],
                     "cuDNN": os.environ["JETSON_CUDNN"],
                     "TensorRT": os.environ["JETSON_TENSORRT"],
                     "VisionWorks": os.environ["JETSON_VISIONWORKS"],
                     "OpenCV": os.environ["JETSON_OPENCV"] + " compiled CUDA: " + os.environ["JETSON_OPENCV_CUDA"]}
        return {"board": board, "libraries": libraries}

    @property
    def stats(self):
        # Wait the deque not empty
        while not self._stats:
            pass
        # Return dictionary parsed
        return self._stats

    def open(self):
        try:
            self.tegrastats.open(self.reader)
        except Tegrastats.TegrastatsException as e:
            raise jtop.JtopException(e)

    def close(self):
        self.tegrastats.close()

    def reader(self, stats):
        # Update status
        self._stats = stats
        # Update nvpmodel
        if self.nvp is not None:
            self.nvp.update()
        # Update status from fan
        if self.qfan is not None:
            self.qfan.update()

    def __enter__(self):
        self.open()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()
# EOF



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import curses
# Graphics elements
from .jtopguilib import (check_size,
                         check_curses)


class JTOPGUI:

    def __init__(self, stdscr, refresh, pages, init_page=0):
        self.stdscr = stdscr
        self.pages = pages
        self.n_page = init_page
        self.refresh = refresh
        self.key = -1
        self.old_key = -1

    @check_size(20, 50)
    def draw(self, jetson):
        # First, clear the screen
        self.stdscr.erase()
        # Write head of the jtop
        self.header(jetson)
        # Write page selected
        if "func" in self.pages[self.n_page]:
            page = self.pages[self.n_page]["func"]
            if page is not None:
                page(self.stdscr, jetson, self.key)
        # Draw menu
        self.menu()
        # Draw the screen
        self.stdscr.refresh()
        # Set a timeout and read keystroke
        self.stdscr.timeout(self.refresh)

    def increase(self):
        idx = self.n_page + 1
        self.set(idx + 1)

    def decrease(self):
        idx = self.n_page + 1
        self.set(idx - 1)

    def set(self, idx):
        if idx <= len(self.pages) and idx > 0:
            self.n_page = idx - 1

    @check_curses
    def header(self, jetson):
        board = jetson.board["board"]
        board_info = board["name"] + " - Jetpack " + board["jetpack"]
        self.stdscr.addstr(0, 0, board_info, curses.A_BOLD)
        if jetson.userid != 0:
            self.stdscr.addstr(0, len(board_info) + 1, "- PLEASE RUN WITH SUDO", curses.color_pair(1))

    @check_curses
    def menu(self):
        height, width = self.stdscr.getmaxyx()
        # Set background for all menu line
        self.stdscr.addstr(height - 1, 0, ("{0:<" + str(width - 1) + "}").format(" "), curses.A_REVERSE)
        position = 1
        for idx, page in enumerate(self.pages):
            name = page["name"]
            if self.n_page != idx:
                self.stdscr.addstr(height - 1, position, ("{idx} {name}").format(idx=idx + 1, name=name), curses.A_REVERSE)
            else:
                self.stdscr.addstr(height - 1, position, ("{idx} {name}").format(idx=idx + 1, name=name))
            position += len(name) + 2
            self.stdscr.addstr(height - 1, position, " - ", curses.A_REVERSE)
            position += 3
        # Add close option menu
        self.stdscr.addstr(height - 1, position, "Q to close", curses.A_REVERSE)
        # Author name
        name_author = "Raffaello Bonghi"
        self.stdscr.addstr(height - 1, width - len(name_author), name_author, curses.A_REVERSE)

    def keyboard(self, jetson):
        self.key = self.stdscr.getch()
        if self.old_key != self.key:
            # keyboard check list
            if self.key == curses.KEY_LEFT:
                self.decrease()
            elif self.key == curses.KEY_RIGHT:
                self.increase()
            elif self.key in [ord(str(n)) for n in range(10)]:
                num = int(chr(self.key))
                self.set(num)
            elif self.key == ord('q') or self.key == ord('Q'):
                # keyboard check quit button
                return False
            else:
                # Run extra control for page if exist
                for page in self.pages:
                    if "key" in page:
                        # Run key controller
                        page["key"](self.stdscr, jetson, self.key)
            # Store old value key
            self.old_key = self.key
        return True
# EOF



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Logging
import logging
# Launch command
import subprocess as sp

# Create logger for jplotlib
logger = logging.getLogger(__name__)


class JetsonClocks(object):

    def __init__(self, service='jetson_performance'):
        self.service = service
        self.last_status = ""

    @property
    def status(self):
        p = sp.Popen(['systemctl', 'is-active', self.service + '.service'], stdout=sp.PIPE)
        out, _ = p.communicate()
        return str(out.decode("utf-8")).strip()

    @property
    def start(self):
        start_val = True if self.status == "active" else False
        return start_val

    @start.setter
    def start(self, value):
        if not isinstance(value, bool):
            raise Exception("Use a boolean")
        start_val = "start" if value else "stop"
        p = sp.Popen(['systemctl', start_val, self.service + '.service'], stdout=sp.PIPE, stderr=sp.PIPE)
        _, err = p.communicate()
        self.last_status = err.decode("utf-8")

    @property
    def enable(self):
        p = sp.Popen(['systemctl', 'is-enabled', self.service + '.service'], stdout=sp.PIPE)
        out, _ = p.communicate()
        enable_val = True if str(out.decode("utf-8")).strip() == "enabled" else False
        return enable_val

    @enable.setter
    def enable(self, value):
        if not isinstance(value, bool):
            raise Exception("Use a boolean")
        enable_val = "enable" if value else "disable"
        p = sp.Popen(['systemctl', enable_val, self.service + '.service'], stdout=sp.PIPE, stderr=sp.PIPE)
        _, err = p.communicate()
        self.last_status = err.decode("utf-8")

# EOF



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
    Graphic reference:
    - http://urwid.org/examples/index.html
    - https://npyscreen.readthedocs.io/
    - https://github.com/chubin/cheat.sh
    - https://stackoverflow.com/questions/6840420/python-rewrite-multiple-lines-in-the-console
    - https://docs.python.org/3.3/howto/curses.html#attributes-and-color
    - http://toilers.mines.edu/~jrosenth/101python/code/curses_plot/
"""
import os
import sys
import argparse
import time
# Logging
import logging
# control command line
import curses
# Tegrastats objext reader
from .jtop import jtop
# GUI jtop interface
from .gui import JTOPGUI, all_info, GPU, CTRL, CTRL_keyboard, Variables

# Create logger for jplotlib
logger = logging.getLogger(__name__)


def set_xterm_title(title):
    '''
    Set XTerm title using escape sequences.
    By default, sets as 'Python' and the version number.
    '''
    # Make sure this terminal supports the OSC code (\33]),
    # though not necessarily that it supports setting the title.
    # If this check causes compatibility issues, you can add
    # items to the tuple, or remove the check entirely.
    if os.environ.get('TERM') in ('xterm',
                                  'xterm-color',
                                  'xterm-256color',
                                  'linux',
                                  'screen',
                                  'screen-256color',
                                  'screen-bce',
                                  ):
        sys.stdout.write('\33]0;' + title + '\a')
        sys.stdout.flush()


# The easiest way to use curses is to use a wrapper around a main function
# Essentially, what goes in the main function is the body of your program,
# The `stdscr' parameter passed to it is the curses screen generated by our
# wrapper.
def gui(stdscr, args, jetson):
    # In this program, we don't want keystrokes echoed to the console,
    # so we run this to disable that
    curses.noecho()
    # Additionally, we want to make it so that the user does not have to press
    # enter to send keys to our program, so here is how we get keys instantly
    curses.cbreak()
    # Define pairing colors
    curses.init_pair(1, curses.COLOR_RED, curses.COLOR_BLACK)
    curses.init_pair(2, curses.COLOR_GREEN, curses.COLOR_BLACK)
    curses.init_pair(3, curses.COLOR_YELLOW, curses.COLOR_BLACK)
    curses.init_pair(4, curses.COLOR_BLUE, curses.COLOR_BLACK)
    curses.init_pair(5, curses.COLOR_MAGENTA, curses.COLOR_BLACK)
    curses.init_pair(6, curses.COLOR_CYAN, curses.COLOR_BLACK)
    # Hide the cursor
    curses.curs_set(0)
    # Lastly, keys such as the arrow keys are sent as funny escape sequences to
    # our program. We can make curses give us nicer values (such as curses.KEY_LEFT)
    # so it is easier on us.
    stdscr.keypad(True)
    # Refreshing page curses loop
    # https://stackoverflow.com/questions/54409978/python-curses-refreshing-text-with-a-loop
    stdscr.nodelay(1)
    # Initialize key
    # Initialization Menu
    pages = JTOPGUI(stdscr, args.refresh, [{"name": "ALL", "func": all_info},
                                           {"name": "GPU", "func": GPU},
                                           {"name": "CTRL", "func": CTRL, "key": CTRL_keyboard},
                                           {"name": "INFO", "func": Variables},
                                           ])
    # Start with selected page
    pages.set(args.page)
    # Here is the loop of our program, we keep clearing and redrawing in this loop
    while pages.keyboard(jetson):
        # Draw pages
        pages.draw(jetson)


def main():
    # Add arg parser
    parser = argparse.ArgumentParser(description='jtop is system monitoring utility and runs in the terminal')
    parser.add_argument('-r', dest="refresh", help='refresh interval', type=int, default='500')
    parser.add_argument('--debug', dest="debug", help='Run with debug logger', action="store_true", default=False)
    parser.add_argument('--page', dest="page", help='Open fix page', type=int, default=1)
    # Parse arguments
    args = parser.parse_args()
    # Set logging level
    if args.debug:
        logging.basicConfig(level=logging.DEBUG, filename='jtop.log', filemode='w',
                            format='%(name)s - %(levelname)s - %(message)s')
    else:
        logging.basicConfig()
    # Title script
    # Reference: https://stackoverflow.com/questions/25872409/set-gnome-terminal-window-title-in-python
    set_xterm_title("jtop")
    # Open tegrastats reader and run the curses wrapper
    try:
        with jtop(interval=args.refresh) as jetson:
            try:
                # Run jtop in two different modes:
                # - Debug mode
                # - GUI mode
                if args.debug:
                    while True:
                        print(jetson.stats)
                        # Sleep before send new stat
                        time.sleep(1)
                else:
                    # Call the curses wrapper
                    curses.wrapper(gui, args, jetson)
            except KeyboardInterrupt:
                # Catch keyboard interrupt and close
                logger.info("Closed with CTRL-C")
    except jtop.JtopException as e:
        # Print error and close
        print(e)


if __name__ == "__main__":
    main()
# EOF



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import curses
# Graphics elements
from .jtopguilib import (check_curses,
                         box_keyboard,
                         box_status,
                         box_list,
                         draw_chart)


@check_curses
def CTRL(stdscr, jetson, key):
    """ Control board, check status jetson_clocks and change NVP model """
    # Screen size
    height, width = stdscr.getmaxyx()
    # Position information
    posx = 2
    start_pos = 2
    stdscr.addstr(start_pos, posx, "jetson_clocks controller", curses.A_BOLD)
    if jetson.userid == 0:
        # button start/stop jetson clocks
        box_keyboard(stdscr, start_pos - 1, posx + 1, "a", key)
    # Read status jetson_clocks
    start = jetson.jetson_clocks.start
    status = jetson.jetson_clocks.status
    box_status(stdscr, start_pos + 4, posx + 1, status.capitalize(), start)
    if jetson.userid == 0:
        # button start/stop jetson clocks
        box_keyboard(stdscr, start_pos - 1, posx + 4, "e", key)
    # Read status jetson_clocks
    enabled = jetson.jetson_clocks.enable
    enabled_box = "Enable" if enabled else "Disable"
    box_status(stdscr, start_pos + 4, posx + 4, enabled_box, enabled)
    # Build NVP model list
    nvpmodel = jetson.nvpmodel
    if nvpmodel is not None:
        stdscr.addstr(start_pos + 8, posx, "NVP model", curses.A_BOLD)
        if jetson.userid == 0:
            # Draw keys to decrease nvpmodel
            box_keyboard(stdscr, start_pos + 10, posx + 7, "-", key)
            # Draw selected number
            stdscr.addstr(start_pos + 8, posx + 16, str(nvpmodel.selected), curses.A_NORMAL)
            # Draw keys to increase nvpmodel
            box_keyboard(stdscr, start_pos + 18, posx + 7, "+", key)
        # Write list of available modes
        mode_names = [mode["Name"] for mode in nvpmodel.modes]
        mode_status = [mode["status"] for mode in nvpmodel.modes]
        box_list(stdscr, start_pos - 1, posx + 10, mode_names, nvpmodel.num, status=mode_status, max_width=42, numbers=True)
    # Add plot fan status
    fan = jetson.fan
    if fan is not None:
        # Evaluate size chart
        size_x = [posx + 40, width - 10]
        size_y = [2, height - 3]
        percent = fan['percent'] if 'percent' in fan else ""
        fan_value = {'name': 'FAN speed', 'percent': percent}
        # If exist value of fan add in fan value
        fan_value['idle'] = fan['value'] if 'value' in fan else [0]
        # Draw the GPU chart
        draw_chart(stdscr, size_x, size_y, fan_value, line="o", color=curses.color_pair(4))


def CTRL_keyboard(stdscr, jetson, key):
    if jetson.userid == 0:
        # Start jetson_clocks
        start = jetson.jetson_clocks.start
        # Status enable
        enabled = jetson.jetson_clocks.enable
        # NVPmodel controller
        nvpmodel = jetson.nvpmodel
        # Write the new jetson_clocks status
        if key == ord('a') and not start:
            jetson.jetson_clocks.start = True
        elif key == ord('a') and start:
            jetson.jetson_clocks.start = False
        # Write the new jetson_clocks status
        elif key == ord('e') and not enabled:
            jetson.jetson_clocks.enable = True
        elif key == ord('e') and enabled:
            jetson.jetson_clocks.enable = False
        # Enable nvpmodel control
        if nvpmodel is not None:
            if key == ord('+'):
                nvpmodel.increase()
            elif key == ord('-'):
                nvpmodel.decrease()
# EOF



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import curses
from datetime import timedelta
# Graphics elements
from .jtopguilib import (check_curses,
                         plot_name_info)
# Menu GUI pages
from .jtopguimenu import strfdelta


@check_curses
def Variables(stdscr, jetson, key):
    """
        Write all environment variables
    """
    # Screen size
    height, width = stdscr.getmaxyx()
    # Position information
    posx = 2
    start_pos = 2
    spacing = 20
    # Up time
    uptime_string = strfdelta(timedelta(seconds=jetson.uptime), "{days} days {hours}:{minutes}:{seconds}")
    plot_name_info(stdscr, start_pos, posx, "- Up Time", uptime_string)
    start_pos += 1
    # Loop build information
    idx = 0
    # Board info
    stdscr.addstr(start_pos + idx, posx, "- Board:", curses.A_BOLD)
    for name, info in jetson.board["board"].items():
        stdscr.addstr(start_pos + idx + 1, posx + 2, "* " + name + ":")
        stdscr.addstr(start_pos + idx + 1, posx + spacing, info, curses.A_BOLD)
        idx += 1
    # Libraries info
    stdscr.addstr(start_pos + idx + 1, posx, "- Libraries:", curses.A_BOLD)
    idx += 1
    for name, info in jetson.board["libraries"].items():
        stdscr.addstr(start_pos + idx + 1, posx + 2, "* " + name + ":")
        stdscr.addstr(start_pos + idx + 1, posx + spacing, info, curses.A_BOLD)
        idx += 1
    # IP address and Hostname
    if jetson.local_interfaces:
        plot_name_info(stdscr, start_pos + idx + 1, posx, "- Hostname", jetson.local_interfaces["hostname"])
        stdscr.addstr(start_pos + idx + 2, posx, "- Interfaces", curses.A_BOLD)
        idx += 3
        for name, ip in jetson.local_interfaces["interfaces"].items():
            stdscr.addstr(start_pos + idx, posx + 2, "* " + name + ":")
            stdscr.addstr(start_pos + idx, posx + spacing, ip, curses.A_BOLD)
            idx += 1
    # Author information
    plot_name_info(stdscr, start_pos, width - 30, "Author", "Raffaello Bonghi")
    plot_name_info(stdscr, start_pos + 1, width - 30, "e-mail", "raffaello@rnext.it")



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
# Logging
import logging
# Launch command
import subprocess as sp
from collections import deque

# Create logger for jplotlib
logger = logging.getLogger(__name__)


class Fan(object):

    class FanException(Exception):
        pass

    def __init__(self, path, interval, time):
        # Initialize number max records to record
        self.path = path
        # Check exist path
        if not os.path.isdir(path):
            raise Fan.FanException("Fan does not exist")
        # Init variables
        max_record = int(float(time) * (float(1 / float(interval)) * 1000.0))
        self.fan_ctrl = deque(max_record * [0], maxlen=max_record)
        self.fan_read = deque(max_record * [0], maxlen=max_record)
        self.rpm = deque(max_record * [0], maxlen=max_record)
        self.with_rpm = os.path.isfile(self.path + "rpm_measured")
        self.with_curr = os.path.isfile(self.path + "cur_pwm")
        self.with_target = os.path.isfile(self.path + "target_pwm")
        # Max value PWM
        if os.path.isfile(self.path + "pwm_cap"):
            self.pwm_max = int(self.read_status("pwm_cap"))
        else:
            self.pwm_max = 255
        # PWM RPM table
        if os.path.isfile(self.path + "pwm_rpm_table"):
            pwm_rpm_table = self.read_status("pwm_rpm_table")
            for line in pwm_rpm_table.decode("utf-8").split("\n"):
                line = [val.strip() for val in line.split(",")]
                # print(line)
        # Step time
        if os.path.isfile(self.path + "step_time"):
            self.step_time = int(self.read_status("step_time"))
        else:
            self.step_time = 0
        # Control temperature
        if os.path.isfile(self.path + "temp_control"):
            self.temp_control = int(self.read_status("temp_control"))
        # Status FAN
        if os.getuid() == 0:
            self.qstatus = 'ON' if os.path.isfile(self.path + "target_pwm") else 'OFF'
        else:
            self.qstatus = 'REQUIRE SUDO'

    def read_status(self, file_read):
        status = sp.Popen(['cat', self.path + file_read], stdout=sp.PIPE)
        query, _ = status.communicate()
        return query

    def update(self):
        # Read PWM
        if self.with_target:
            fan_level = float(self.read_status("target_pwm")) / 255.0 * 100.0
            logger.debug('{} status PWM CTRL {}'.format(self.path, fan_level))
            self.fan_ctrl.append(int(fan_level))
        # Read current
        if self.with_curr:
            fan_level = float(self.read_status("cur_pwm")) / 255.0 * 100.0
            logger.debug('{} status PWM CUR {}'.format(self.path, fan_level))
            self.fan_read.append(int(fan_level))
        # Read RPM fan
        # if self.with_rpm:
        #     rpm_measured = int(self.read_status("rpm_measured"))
        #     logger.debug('{} status RPM {}'.format(self.path, rpm_measured))
        #     self.rpm.append(rpm_measured)

    @property
    def status(self):
        fan = {'name': 'FAN',
               'status': self.qstatus
               }
        if self.with_curr:
            fan['value'] = list(self.fan_read)
            fan['percent'] = str(self.fan_read[-1]) + "/" + str(self.fan_ctrl[-1]) + "%"
        elif self.with_target:
            fan['value'] = list(self.fan_ctrl)
        # TODO Improve RPM read
        # if self.with_rpm and self.rpm[-1] != 0:
        #     fan['label'] = str(self.rpm[-1]) + "RPM"
        return fan



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import re
import os
# Logging
import logging
# Launch command
import subprocess as sp
# Threading
from threading import Thread
from collections import deque

# Create logger for jplotlib
logger = logging.getLogger(__name__)


def UpdateProcess(text, status):
    status['idle'].pop(0)
    status['frequency'].pop(0)
    if 'off' in text:
        status['idle'].append(0.0)
        status['frequency'].append(0.0)
        status['status'] = "OFF"
    elif '@' in text:
        info = re.findall(r"\d+", text)
        status['idle'].append(float(info[0]))
        status['frequency'].append(float(info[1]))
        status['status'] = "ON"
    else:
        info = re.findall(r"\d+", text)
        status['idle'].append(float(info[0]))
        status['frequency'].append(0.0)
        status['status'] = "ON-SIMPLE"
    return status


def InitProcess(name, max_record, status="OFF"):
    return {'name': name,
            'idle': max_record * [0.0],
            'frequency': max_record * [0.0],
            'status': status,
            }


class Tegrastats(Thread):
    """
        - Subprocess read:
        https://stackoverflow.com/questions/375427/non-blocking-read-on-a-subprocess-pipe-in-python/4896288#4896288
        - Property
        https://www.programiz.com/python-programming/property
    """

    class TegrastatsException(Exception):
        pass

    def __init__(self, path, interval, time):
        Thread.__init__(self)
        self.interval = interval
        # Initialize number max records to record
        max_record = int(float(time) * (float(1 / float(interval)) * 1000.0))
        self.max_record = max_record
        # Sensors deque list
        self.swap = deque(max_record * [0], maxlen=max_record)
        self.iram = deque(max_record * [0], maxlen=max_record)
        self.ram = deque(max_record * [0], maxlen=max_record)
        self.cpus = {}
        self.emc = InitProcess("EMC", max_record, status="REQUIRE SUDO")
        self.gpu = InitProcess("GPU", max_record, status="REQUIRE SUDO")
        self.temperatures = {}
        self.voltages = {}
        # Initialize jetson stats
        self._jetsonstats = {}
        # Start process tegrastats
        self.path = path
        # Callback function
        self.callback = None
        # Define Tegrastats process
        self.p = None

    def run(self):
        try:
            while self.p.poll() is None:
                out = self.p.stdout
                if out is not None:
                    # Read line process output
                    line = out.readline()
                    # Decode line in UTF-8
                    tegrastats_data = line.decode("utf-8")
                    # Decode and store
                    stats = self.decode(tegrastats_data)
                    # If callback is defined after each decode will be send the updates by function
                    if self.callback is not None:
                        self.callback(stats)
                    else:
                        self._jetsonstats = stats
        except SystemExit:
            logger.error("System exit", exc_info=True)
        except AttributeError:
            logger.error("Attribute error", exc_info=True)

    @property
    def stats(self):
        # Wait the deque not empty
        while not self._jetsonstats:
            pass
        # Return dictionary parsed
        return self._jetsonstats

    def open(self, callback=None):
        try:
            # Launch subprocess or raise and exception
            self.p = sp.Popen([self.path, '--interval', str(self.interval)], stdout=sp.PIPE)
            # Start himself like file
            self.daemon = True
            self.start()
            # If callback is defined after each decode will be send the updates by function
            self.callback = callback
            return True
        except OSError:
            logger.error("Tegrastats not in list!")
            raise Tegrastats.TegrastatsException("Tegrastats is not available on this hardware")
        return False

    def close(self):
        if self.p is not None:
            self.p.kill()
            return True
        else:
            return False

    def __enter__(self):
        self.open()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def _SWAP(self, text):
        # SWAP X/Y (cached Z)
        # X = Amount of SWAP in use in megabytes.
        # Y = Total amount of SWAP available for applications.
        # Z = Amount of SWAP cached in megabytes.
        find_swap = re.search(r'SWAP (.+?)B \(cached (.+?)B\)', text)
        if find_swap is not None:
            swap_string = find_swap.group()
            swap_stat = re.findall(r"\d+", swap_string)
            text = re.sub(r'SWAP (.+?)B \(cached (.+?)B\) ', '', text)
            self.swap.append(float(swap_stat[0]))
            return {'used': list(self.swap),
                    'total': float(swap_stat[1]),
                    'cached': int(swap_stat[2])
                    }, text
        else:
            return {}, text

    def _IRAM(self, text):
        # IRAM X/Y (lfb Z)
        # IRAM is memory local to the video hardware engine.
        # X = Amount of IRAM memory in use, in kilobytes.
        # Y = Total amount of IRAM memory available.
        # Z = Size of the largest free block.
        find_iram = re.search(r'IRAM (.+?)B\(lfb (.+?)B\)', text)
        # Find if IRAM is inside
        if find_iram is not None:
            iram_lfb_string = find_iram.group()
            iram_stat = re.findall(r"\d+", iram_lfb_string)
            text = re.sub(r'IRAM (.+?)B\(lfb (.+?)B\) ', '', text)
            self.iram.append(float(iram_stat[0]))
            return {'used': list(self.iram),
                    'total': float(iram_stat[1]),
                    'size': int(iram_stat[2])
                    }, text
        else:
            return {}, text

    def _RAM(self, text):
        # RAM X/Y (lfb NxZ)
        # Largest Free Block (lfb) is a statistic about the memory allocator.
        # It refers to the largest contiguous block of physical memory
        # that can currently be allocated: at most 4 MB.
        # It can become smaller with memory fragmentation.
        # The physical allocations in virtual memory can be bigger.
        # X = Amount of RAM in use in MB.
        # Y = Total amount of RAM available for applications.
        # N = The number of free blocks of this size.
        # Z = is the size of the largest free block.
        ram_search = re.search(r'RAM (.+?)B', text)
        lfb_search = re.search(r'\(lfb (.+?)\)', text)
        if ram_search is not None and lfb_search is not None:
            ram_string = ram_search.group()
            lfb_string = lfb_search.group()
            ram_stat = re.findall(r"\d+", ram_string)
            lfb_stat = re.findall(r"\d+", lfb_string)
            text = re.sub(r'RAM (.+?)\) ', '', text)
            self.ram.append(float(ram_stat[0]))
            return {'RAM': {'used': list(self.ram), 'total': float(ram_stat[1])},
                    'lfb': {'nblock': lfb_stat[0], 'size': lfb_stat[1]},
                    }, text
        else:
            return {}, text

    def _CPU(self, text):
        # CPU [X%,Y%, , ]@Z
        # or
        # CPU [X%@Z, Y%@Z,...]
        # X and Y are rough approximations based on time spent
        # in the system idle process as reported by the Linux kernel in /proc/stat.
        # X = Load statistics for each of the CPU cores relative to the
        #     current running frequency Z, or 'off' in case a core is currently powered down.
        # Y = Load statistics for each of the CPU cores relative to the
        #     current running frequency Z, or 'off' in case a core is currently powered down.
        # Z = CPU frequency in megahertz. Goes up or down dynamically depending on the CPU workload.
        cpu_search = re.search(r'CPU (.+?)\]', text)
        if cpu_search is not None:
            cpu_string = cpu_search.group()
            cpu_string = cpu_string[cpu_string.find("[") + 1:cpu_string.find("]")]
            text = re.sub(r'CPU (.+?)\] ', '', text)
            for idx, cpu in enumerate(cpu_string.split(",")):
                name = "CPU" + str(idx + 1)
                if idx in self.cpus:
                    cpu_status = self.cpus[idx]
                else:
                    cpu_status = InitProcess(name, self.max_record)
                cpu_status = UpdateProcess(cpu, cpu_status)
                # Update status governor
                governor_name = '/sys/devices/system/cpu/cpu' + str(idx) + '/cpufreq/scaling_governor'
                if os.path.isfile(governor_name):
                    with open(governor_name, 'r') as f:
                        cpu_status['governor'] = f.read()[:-1]
                else:
                    cpu_status['governor'] = ""
                # Update status CPU
                self.cpus[idx] = cpu_status
            return text
        else:
            return text

    def decode(self, text):
        jetsonstats = {}
        # Read SWAP status
        swap_status, text = self._SWAP(text)
        jetsonstats['SWAP'] = swap_status
        # Read IRAM status
        iram_status, text = self._IRAM(text)
        jetsonstats['IRAM'] = iram_status
        # Read RAM status
        ram_status, text = self._RAM(text)
        jetsonstats['RAM'] = ram_status
        # Read CPU status
        text = self._CPU(text)
        jetsonstats['CPU'] = list(self.cpus.values())
        # Start while loop to decode
        idx = 0
        other_values = text.split(" ")
        while idx < len(other_values):
            data = other_values[idx]
            if 'EMC' in data:
                # EMC X%@Y
                # EMC is the external memory controller,
                # through which all sysmem/carve-out/GART memory accesses go.
                # X = Percent of EMC memory bandwidth being used, relative to the current running frequency.
                # Y = EMC frequency in megahertz.
                self.emc = UpdateProcess(other_values[idx + 1], self.emc)
                # extra increase counter
                idx += 1
            elif 'APE' in data:
                # APE Y
                # APE is the audio processing engine.
                # The APE subsystem consists of ADSP (Cortex®-A9 CPU), mailboxes, AHUB, ADMA, etc.
                # Y = APE frequency in megahertz.
                jetsonstats['APE'] = other_values[idx + 1]
                # extra increase counter
                idx += 1
            elif 'GR3D' in data:
                # GR3D X%@Y
                # GR3D is the GPU engine.
                # X = Percent of the GR3D that is being used, relative to the current running frequency.
                # Y = GR3D frequency in megahertz
                self.gpu = UpdateProcess(other_values[idx + 1], self.gpu)
                # extra increase counter
                idx += 1
            elif 'MTS' in data:
                # MTS fg X% bg Y%
                # X = Time spent in foreground tasks.
                # Y = Time spent in background tasks.
                fg = float(other_values[idx + 2].split("%")[0])
                bg = float(other_values[idx + 4].split("%")[0])
                jetsonstats['MTS'] = {'fg': fg, 'bg': bg}
                # extra increase counter
                idx += 4
            elif 'MSENC' in data:
                # MSENC Y
                # Y = MSENC frequency in megahertz.
                # MSENC is the video hardware encoding engine.
                jetsonstats['MSENC'] = int(other_values[idx + 1])
                # extra increase counter
                idx += 1
            elif 'NVDEC' in data:
                # NVDEC Y
                # Y = NVDEC frequency in megahertz.
                # NVDEC is the video hardware decoding engine.
                # It is shown only when hardware decoder/encoder engine is used.
                jetsonstats['NVDEC'] = int(other_values[idx + 1])
                # extra increase counter
                idx += 1
            elif '@' in data:
                # [temp name] C
                # [temp name] is one of the names under the nodes
                # /sys/devices/virtual/thermal/thermal_zoneX/type.
                info = data.split("@")
                name = info[0].strip()
                value = info[1]
                # Read from dictionary temperature or initialize it
                if name in self.temperatures:
                    temp = self.temperatures[name]
                else:
                    temp = {'value': self.max_record * [0.0], 'unit': 'C'}
                # Remove last measured temp
                temp['value'].pop(0)
                # Add new value
                temp['value'].append(float(value.split("C")[0]))
                # Update text value
                temp['text'] = value
                # Store temperature value
                self.temperatures[name] = temp
            elif idx + 1 < len(other_values):
                # [VDD_name] X/Y
                # X = Current power consumption in milliwatts.
                # Y = Average power consumption in milliwatts.
                value = other_values[idx + 1].split("/")
                # Read from dictionary temperature or initialize it
                if data in self.voltages:
                    volt = self.voltages[data]
                else:
                    volt = {'current': self.max_record * [0.0], 'average': self.max_record * [0.0], 'unit': 'mW'}
                # Remove last measure
                volt['current'].pop(0)
                volt['average'].pop(0)
                # Add new value
                volt['current'].append(float(value[0]))
                volt['average'].append(float(value[1]))
                # Update text value
                volt['text'] = other_values[idx + 1].rstrip() + "mW"
                # Store temperature value
                self.voltages[data] = volt
                # extra increase counter
                idx += 1
            # Update counter
            idx += 1
        # EMC status
        jetsonstats['EMC'] = self.emc
        # Add GPU status
        jetsonstats['GR3D'] = self.gpu
        # Add Temperatures and voltages
        jetsonstats['temperatures'] = self.temperatures
        jetsonstats['voltages'] = self.voltages
        return jetsonstats
# EOF



# -*- coding: UTF-8 -*-
# Copyright (C) 2019, Raffaello Bonghi <raffaello@rnext.it>
# All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import curses
# Graphics elements
from .jtopguilib import (linear_percent_gauge,
                         make_gauge_from_percent)
# Menu GUI pages
from .jtopguimenu import (plot_voltages,
                          compact_info,
                          plot_temperatures,
                          plot_CPUs)


def all_info(stdscr, jetson, key):
    """
        Update screen with values
    """
    # Screen size
    height, width = stdscr.getmaxyx()
    line_counter = 1
    # Plot Status CPU
    line_counter = plot_CPUs(stdscr, line_counter, jetson.stats['CPU'], width)
    # Plot MTS
    if 'MTS' in jetson.stats:
        line_counter += 1
        stdscr.addstr(line_counter, 0, "MTS ", curses.color_pair(5))
        MTS_FG = {'name': 'FG',
                  'value': int(jetson.stats['MTS']['fg']),
                  }
        linear_percent_gauge(stdscr, MTS_FG, width // 2 - 2,
                             offset=line_counter, start=4, color_name=5)
        MTS_BG = {'name': 'BG',
                  'value': int(jetson.stats['MTS']['bg']),
                  }
        linear_percent_gauge(stdscr, MTS_BG, width // 2 - 2,
                             offset=line_counter, start=2 + width // 2, color_name=5)
    # RAM linear gauge info
    ram_status = jetson.stats['RAM']['RAM']
    lfb_status = jetson.stats['RAM']['lfb']
    RAM_VALUE = {'name': "Mem",
                 'value': int(ram_status['used'][-1] / float(ram_status['total']) * 100.0),
                 'label': "(lfb " + str(lfb_status['nblock']) + "x" + str(lfb_status['size']) + "MB)",
                 'percent': "{0:2.1f}GB/{1:2.1f}GB".format(ram_status['used'][-1] / 1000.0, ram_status['total'] / 1000.0),
                 }
    line_counter += 1
    linear_percent_gauge(stdscr, RAM_VALUE, width, offset=line_counter)
    # EMC linear gauge info
    if 'EMC' in jetson.stats:
        line_counter += 1
        linear_percent_gauge(stdscr, make_gauge_from_percent(jetson.stats['EMC']), width, offset=line_counter)
    # IRAM linear gauge info
    iram_status = jetson.stats['IRAM']
    if iram_status:
        line_counter += 1
        IRAM_VALUE = {'name': "Imm",
                      'value': int(iram_status['used'][-1] / float(iram_status['total']) * 100.0),
                      'label': "(lfb " + str(iram_status['size']) + "MB)",
                      'percent': "{0:2.1f}GB/{1:2.1f}GB".format(iram_status['used'][-1] / 1000.0,
                                                                iram_status['total'] / 1000.0),
                      }
        linear_percent_gauge(stdscr, IRAM_VALUE, width, offset=line_counter)
    # SWAP linear gauge info
    swap_status = jetson.stats['SWAP']
    if swap_status:
        SWAP_VALUE = {'name': "Swp",
                      'value': int(swap_status['used'][-1] / float(swap_status['total']) * 100.0),
                      'label': "(cached " + str(swap_status['cached']) + "MB)",
                      'percent': "{0:2.1f}GB/{1:2.1f}GB".format(swap_status['used'][-1] / 1000.0,
                                                                swap_status['total'] / 1000.0),
                      }
    else:
        SWAP_VALUE = {'name': "Swp"}
    line_counter += 1
    linear_percent_gauge(stdscr, SWAP_VALUE, width, offset=line_counter)
    # GPU linear gauge info
    line_counter += 1
    if 'GR3D' in jetson.stats:
        linear_percent_gauge(stdscr, make_gauge_from_percent(jetson.stats['GR3D']), width, offset=line_counter + 1)
    line_counter += 2
    # Status disk
    disk_status = jetson.disk
    DISK_STATUS = {'name': "Dsk",
                   'value': int(float(disk_status['used']) / float(disk_status['total']) * 100.0),
                   'percent': "{0:2.1f}GB/{1:2.1f}GB".format(disk_status['used'], disk_status['total']),
                   }
    linear_percent_gauge(stdscr, DISK_STATUS, width, offset=line_counter, type_bar="#", color_name=3)
    # Last part of information
    split = 1.0
    split += 1.0 if jetson.stats['temperatures'] else 0.0
    split += 1.0 if jetson.stats['voltages'] else 0.0
    column_width = int(float(width - 4) / split)
    line_counter += 1
    # List of all mini menu
    mini_menu = [compact_info, plot_temperatures, plot_voltages]
    # Evaluate column width
    column_width = int(float(width) / len(mini_menu))
    for idx, mini in enumerate(mini_menu):
        # Run mini page
        mini(stdscr, idx * column_width, line_counter, column_width, jetson)
